@@ -6,7 +6,7 @@ make instances.  For details, see Scripting in the reference
 documentation.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -43,7 +43,7 @@ def _getSQLScriptGrammar():
 		comment = sqlComment | cStyleComment
 		lineEnd = Literal("\n")
 
-		simpleStr = QuotedString(quoteChar="'", escChar="\\", 
+		simpleStr = QuotedString(quoteChar="'", escChar="\\",
 			multiline=True, unquoteResults=False)
 		quotedId = QuotedString(quoteChar='"', escChar="\\", unquoteResults=False)
 		dollarQuoted = Regex(r"(?s)\$(\w*)\$.*?\$\1\$")
@@ -100,7 +100,7 @@ class ScriptRunner(object):
 		self._prepare(script)
 	
 	def _prepare(self, script):
-		raise ValueError("Cannot instantate plain ScriptRunners")
+		raise ValueError("Cannot instantiate plain ScriptRunners")
 
 
 class SQLScriptRunner(ScriptRunner):
@@ -111,7 +111,7 @@ class SQLScriptRunner(ScriptRunner):
 	Keyword arguments to run are ignored.
 	"""
 	def _prepare(self, script):
-		self.statements = utils.pyparseString(getSQLScriptGrammar(), 
+		self.statements = utils.pyparseString(getSQLScriptGrammar(),
 			script.getSource())
 	
 	def run(self, dbTable, **kwargs):
@@ -214,7 +214,7 @@ class Script(base.Structure, base.RestrictionMixin):
 	typeDesc_ = "Embedded executable code with a type definition"
 
 	_lang = base.EnumeratedUnicodeAttribute("lang", default=base.Undefined,
-		description="Language of the script.", 
+		description="Language of the script.",
 		validValues=RUNNER_CLASSES.keys(), copyable=True)
 	_type = base.EnumeratedUnicodeAttribute("type", default=base.Undefined,
 		description="Point of time at which script is to run (not all"
@@ -222,7 +222,7 @@ class Script(base.Structure, base.RestrictionMixin):
 		validValues=["preImport", "newSource", "preIndex", "preCreation",
 			"postCreation", "afterMeta",
 			"beforeDrop", "sourceDone"], copyable=True)
-	_name = base.UnicodeAttribute("name", default="anonymous",
+	_name = base.UnicodeAttribute("name", default=base.NotGiven,
 		description="A human-consumable designation of the script.",
 		copyable=True)
 	_notify = base.BooleanAttribute("notify", default=True,
@@ -241,10 +241,18 @@ class Script(base.Structure, base.RestrictionMixin):
 			raise base.StructureError("Invalid script type %s for %s elements"%(
 				self.type, self.parent.name_))
 
-		self._validateNext(Script)
+		super().validate()
+
+	def completeElement(self, ctx):
+		if self.name is base.NotGiven:
+			if self.id:
+				self.name = self.id
+			else:
+				self.name = "anonymous"
+		super().completeElement(ctx)
 
 
-class ScriptingMixin(object):
+class ScriptingMixin(base.StructCallbacks):
 	"""A mixin that gives objects a getRunner method and a script attribute.
 
 	The getRunner() method returns a callable that takes the current table
