@@ -10,7 +10,7 @@ In the DaCHS, a resource descriptor typically sets up a schema in
 the database.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -36,6 +36,28 @@ from gavo.rscdef import scripting
 from gavo.rscdef import executing
 
 
+class ResdirAttribute(base.UnicodeAttribute):
+	"""An attribute representing an RD's resdir.
+
+	This is for resource's resdir attribute, which will by default
+	get the inputsDir prepended.  There is the special value "." that
+	means "the directory containing the RD", which is what people should
+	do now.
+
+	This will only work for RDs.
+
+	Note that without that, the resdir defaults to <inputsDir>/<schemaName>.
+	"""
+	def feedObject(self, instance, parsedValue):
+		setattr(self, f"_original_resdir_{self.name_}", parsedValue)
+		if parsedValue==".":
+			parsedValue = "/".join(instance.sourceId.split("/")[:-1])
+
+		parsedValue = os.path.join(
+			base.getConfig("inputsDir"), parsedValue)
+		super().feedObject(instance, parsedValue)
+
+
 class RD(base.Structure, base.ComputedMetaMixin, scripting.ScriptingMixin,
 		base.StandardMacroMixin, common.PrivilegesMixin):
 	"""A resource descriptor.
@@ -57,14 +79,15 @@ class RD(base.Structure, base.ComputedMetaMixin, scripting.ScriptingMixin,
 	# scripts for a while
 	acceptedScriptTypes = {}
 
-	_resdir = base.FunctionRelativePathAttribute("resdir", 
-		default=None, 
-		baseFunction=lambda instance: base.getConfig("inputsDir"),
+	_resdir = ResdirAttribute("resdir",
+		default=None,
 		description="Base directory for source files and everything else"
-			" belonging to the resource.", 
+			" belonging to the resource.  Use a single dot (.) to say 'the"
+			" directory the RD resides in', which is recommended in modern"
+			" DaCHS.",
 		copyable=True)
 
-	_schema = base.UnicodeAttribute("schema", 
+	_schema = base.UnicodeAttribute("schema",
 		default=base.Undefined,
 		description="Database schema for tables defined here.  Follow the rule"
 		" 'one schema, one RD' if at all possible.  If two RDs share the same"
@@ -74,41 +97,41 @@ class RD(base.Structure, base.ComputedMetaMixin, scripting.ScriptingMixin,
 		copyable=True,
 		callbacks=["_inferResdir"])
 
-	_dds = base.StructListAttribute("dds", 
+	_dds = base.StructListAttribute("dds",
 		childFactory=rscdef.DataDescriptor,
 		description="Descriptors for the data generated and/or published"
-		" within this resource.", 
-		copyable=True, 
+		" within this resource.",
+		copyable=True,
 		before="outputTables")
 
 	_tables = base.StructListAttribute("tables",
-		childFactory=rscdef.TableDef, 
-		description="A table used or created by this resource", 
-		copyable=True, 
+		childFactory=rscdef.TableDef,
+		description="A table used or created by this resource",
+		copyable=True,
 		before="dds")
 
 	_outputTables = base.StructListAttribute("outputTables",
-		childFactory=svcs.OutputTableDef, 
-		description="Canned output tables for later reference.", 
+		childFactory=svcs.OutputTableDef,
+		description="Canned output tables for later reference.",
 		copyable=True)
 
 	_rowmakers = base.StructListAttribute("rowmakers",
-		childFactory=rscdef.RowmakerDef, 
+		childFactory=rscdef.RowmakerDef,
 		description="Transformations for going from grammars to tables."
 			" If specified in the RD, they must be referenced from make"
 			" elements to become active.",
-		copyable=True, 
+		copyable=True,
 		before="dds")
 
-	_procDefs = base.StructListAttribute("procDefs", 
+	_procDefs = base.StructListAttribute("procDefs",
 		childFactory=rscdef.ProcDef,
-		description="Procedure definintions (rowgens, rowmaker applys)",
+		description="Procedure definintions (rowgens, rowmaker apply-s)",
 		copyable=True, before="rowmakers")
 
-	_condDescs = base.StructListAttribute("condDescs", 
+	_condDescs = base.StructListAttribute("condDescs",
 		childFactory=svcs.CondDesc,
-		description="Global condition descriptors for later reference", 
-		copyable=True, 
+		description="Global condition descriptors for later reference",
+		copyable=True,
 		before="cores")
 
 	_resRecs = base.StructListAttribute("resRecs",
@@ -116,25 +139,25 @@ class RD(base.Structure, base.ComputedMetaMixin, scripting.ScriptingMixin,
 		description="Non-service resources for the IVOA registry.  They will"
 			" be published when gavo publish is run on the RD.")
 
-	_services = base.StructListAttribute("services", 
-		childFactory=svcs.Service, 
-		description="Services exposing data from this resource.", 
+	_services = base.StructListAttribute("services",
+		childFactory=svcs.Service,
+		description="Services exposing data from this resource.",
 		copyable=True)
 
-	_macDefs = base.MacDefAttribute(before="tables", 
+	_macDefs = base.MacDefAttribute(before="tables",
 		description="User-defined macros available on this RD")
 
 	_mixinDefs = base.StructListAttribute("mixdefs",
 		childFactory=rscdef.MixinDef,
 		description="Mixin definitions (usually not for users)")
 
-	_require = base.ActionAttribute("require", 
+	_require = base.ActionAttribute("require",
 		methodName="importModule",
 		description="Import the named gavo module (for when you need something"
-		" registred)")
+		" registered)")
 
-	_cores = base.MultiStructListAttribute("cores", 
-		childFactory=svcs.getCore, 
+	_cores = base.MultiStructListAttribute("cores",
+		childFactory=svcs.getCore,
 		childNames=list(svcs.CORE_REGISTRY.keys()),
 		description="Cores available in this resource.", copyable=True,
 		before="services")
@@ -200,7 +223,7 @@ class RD(base.Structure, base.ComputedMetaMixin, scripting.ScriptingMixin,
 			if self.srcPath is not None:
 				return os.path.getmtime(self.srcPath)>self.loadedAt
 		except os.error:
-			# this will ususally mean the file went away
+			# this will usually mean the file went away
 			return True
 		return False
 
@@ -225,9 +248,10 @@ class RD(base.Structure, base.ComputedMetaMixin, scripting.ScriptingMixin,
 			base.ui.notifyWarning("RD %s: resource directory '%s' does not exist"%(
 				self.sourceId, self.resdir))
 
-		self._onElementCompleteNext(RD)
+		super().onElementComplete()
 
 	def _inferResdir(self, value):
+		# a callback for the schema attribute
 		if self.resdir is None:
 			self._resdir.feedObject(self, value)
 
@@ -253,6 +277,13 @@ class RD(base.Structure, base.ComputedMetaMixin, scripting.ScriptingMixin,
 			base.assertType(id, res, forceType)
 		return res
 
+	def getRelResdir(self):
+		"""returns the inputsDir-relative resource directory path.
+
+		This never has either a leading or a trailing path.
+		"""
+		return utils.getRelativePath(self.resdir, base.getConfig("inputsDir"))
+
 	def getAbsPath(self, relPath):
 		"""returns the absolute path for a resdir-relative relPath.
 		"""
@@ -261,7 +292,7 @@ class RD(base.Structure, base.ComputedMetaMixin, scripting.ScriptingMixin,
 	def openRes(self, relPath, mode="rb"):
 		"""returns a file object for relPath within self's resdir.
 
-		Deprecated.  This is going to go away, use getAbsPath and a context 
+		Deprecated.  This is going to go away, use getAbsPath and a context
 		manager.
 		"""
 		return open(self.getAbsPath(relPath), mode)
@@ -277,7 +308,7 @@ class RD(base.Structure, base.ComputedMetaMixin, scripting.ScriptingMixin,
 		"""declares that rd needs the RD prereq to properly work.
 
 		This is used in the generation of resource records to ensure that, e.g.
-		registred data have added their served-bys to the service resources.
+		registered data have added their served-bys to the service resources.
 		"""
 		if rd.sourceId!=prereq.sourceId:
 			self.rdDependencies.add((rd.sourceId, prereq.sourceId))
@@ -295,7 +326,7 @@ class RD(base.Structure, base.ComputedMetaMixin, scripting.ScriptingMixin,
 		See the grammar in //rds for what you can give for reason.
 		"""
 		rsc.makeData(
-			base.resolveCrossId("//rds#update-for-rd"), 
+			base.resolveCrossId("//rds#update-for-rd"),
 			connection=connection,
 			forceSource=(reason, self))
 
@@ -342,8 +373,8 @@ class RD(base.Structure, base.ComputedMetaMixin, scripting.ScriptingMixin,
 		instead of this, as that also sets the rights URI.
 		"""
 		return ("%s is licensed under the `Creative Commons Attribution"
-			" Share-Alike 3.0"
-			" License <http://creativecommons.org/licenses/by-sa/3.0/>`_\n\n"
+			" Share-Alike 4.0"
+			" License <http://creativecommons.org/licenses/by-sa/4.0/>`_\n\n"
 			".. image:: /static/img/ccbysa.png\n   :alt: [CC-BY-SA]\n"
 			)%stuffDesignation
 
@@ -357,8 +388,8 @@ class RD(base.Structure, base.ComputedMetaMixin, scripting.ScriptingMixin,
 		You'll probably want to use the `//procs#license-cc-by`_ stream
 		instead of this, as that also sets the rights URI.
 		"""
-		return ("%s is licensed under the `Creative Commons Attribution 3.0"
-			" License <http://creativecommons.org/licenses/by/3.0/>`_\n\n"
+		return ("%s is licensed under the `Creative Commons Attribution 4.0"
+			" License <http://creativecommons.org/licenses/by/4.0/>`_\n\n"
 			".. image:: /static/img/ccby.png\n   :alt: [CC-BY]\n\n"
 			)%stuffDesignation
 
@@ -388,7 +419,7 @@ class RDParseContext(base.ParseContext):
 
 	It defines a couple of attributes that structures can ask for (however,
 	it's good practice not to rely on their presence in case someone wants
-	to parse XML snippets with a standard parse context, so use 
+	to parse XML snippets with a standard parse context, so use
 	getattr(ctx, "doQueries", True) or somesuch.
 	"""
 	def __init__(self, doQueries=True, restricted=False, forRD=None):
@@ -427,7 +458,7 @@ def canonicalizeRDId(srcId):
 	basically is "inputs-relative path without .rd extension".  Everything
 	that's not within inputs or doesn't end with .rd is handed through.
 	// is expanded to __system__/.  The path to built-in RDs,
-	/resources/inputs, is treated analoguous to inputsDir.
+	/resources/inputs, is treated analogous to inputsDir.
 	"""
 	if srcId.startswith("//"):
 		srcId = "__system__"+srcId[1:]
@@ -485,7 +516,7 @@ def getRDInputStream(srcId):
 	"""
 	for fName in _getFilenamesForId(srcId):
 		if os.path.isfile(fName):
-			# We don't want RDs from outside of inputs and config, as 
+			# We don't want RDs from outside of inputs and config, as
 			# these make referencing really messy.
 			filePath = os.path.abspath(fName)
 			if not (
@@ -498,7 +529,7 @@ def getRDInputStream(srcId):
 
 		if (pkg_resources.resource_exists('gavo', fName)
 				and not pkg_resources.resource_isdir('gavo', fName)):
-			return (PkgResourcePath(fName), 
+			return (PkgResourcePath(fName),
 				pkg_resources.resource_stream('gavo', fName))
 	raise base.RDNotFound(srcId)
 
@@ -506,7 +537,7 @@ def getRDInputStream(srcId):
 def setRDDateTimes(ctx, rd, inputFile):
 	"""sets the _dataUpdated and _metadataUpdated meta items on rd.
 	"""
-	rd.setMeta("_metadataUpdated", 
+	rd.setMeta("_metadataUpdated",
 		datetime.datetime.utcfromtimestamp(
 			utils.fgetmtime(inputFile)))
 
@@ -555,7 +586,7 @@ class _UserConfigFakeRD(object):
 			return base.caches.getRD("//userconfig"
 				).getById(id, forceType=forceType)
 		except base.NotFoundError:
-			raise base.NotFoundError(id, "Element with id", 
+			raise base.NotFoundError(id, "Element with id",
 				"etc/userconfig.rd")
 
 
@@ -604,7 +635,7 @@ def getRD(srcId, doQueries=True, restricted=False, useRD=None):
 			utils.stealVar("getRD_context"), forRD=rd.sourceId)
 	except ValueError:
 		# no getRD_context variable in the stack
-		getRD_context = RDParseContext(doQueries=doQueries, 
+		getRD_context = RDParseContext(doQueries=doQueries,
 			restricted=restricted, forRD=rd.sourceId)
 	
 	rdinj.injectIntoContext(getRD_context, rd.sourceId)
@@ -657,7 +688,7 @@ class CachedException(object):
 			# the RD hasn't been there.  Since we can't tell if the
 			# file has appreared in the mean time, we'll have to re-check
 			return False
-		if not os.path.exists(self.sourcePath): 
+		if not os.path.exists(self.sourcePath):
 			# someone has removed the file, kill cache
 			return True
 		return os.path.getmtime(self.sourcePath)>self.loadedAt
@@ -685,7 +716,7 @@ def _loadRDIntoCache(canonicalRDId, cacheDict):
 
 	if justWait:
 		# Someone else is already parsing.  If it's the current thread,
-		# go on (lock is an RLock!) so we can resolve self-references 
+		# go on (lock is an RLock!) so we can resolve self-references
 		# (as long as they are backward references).  All other threads
 		# just wait for the parsing thread to finish
 		lock.acquire()
@@ -699,7 +730,7 @@ def _loadRDIntoCache(canonicalRDId, cacheDict):
 			# Importing failed, invalidate the RD (in case other threads still
 			# see it from _CURRENTLY_PARSING)
 			if getattr(ex, "cacheable", False):
-				cacheDict[canonicalRDId] = CachedException(ex, 
+				cacheDict[canonicalRDId] = CachedException(ex,
 					getattr(rd, "srcPath", None))
 			rd.invalidate()
 			raise
@@ -739,7 +770,7 @@ def _makeRDCache():
 		clean until it's being changed again.  This will avoid re-parsing
 		RDs all the time when they're broken for a while.
 		"""
-		if (srcId in rdCache 
+		if (srcId in rdCache
 				and getattr(rdCache[srcId], "isDirty", lambda: False)()):
 			try:
 				# limit re-check rate to 10 seconds to avoid getRD storms
@@ -764,7 +795,7 @@ def _makeRDCache():
 				# right (for system and userconfig RDs); but it ought
 				# to be good enough for what users typcially are confronted with.
 				if isinstance(rdCache[srcId], CachedException):
-					rdCache[srcId] = CachedException(ex, 
+					rdCache[srcId] = CachedException(ex,
 						os.path.join(base.getConfig("inputsDir")+srcId+".rd"))
 
 				else:
